@@ -1,4 +1,3 @@
-from __future__ import unicode_literals
 from netmiko.cisco.cisco_ios import (
     CiscoIosBase,
     CiscoIosSSH,
@@ -8,22 +7,32 @@ from netmiko.cisco.cisco_ios import (
 from netmiko.cisco.cisco_ios import CiscoIosFileTransfer
 from netmiko.cisco.cisco_ios import InLineTransfer
 from netmiko.cisco.cisco_asa_ssh import CiscoAsaSSH, CiscoAsaFileTransfer
+from netmiko.cisco.cisco_ftd_ssh import CiscoFtdSSH
 from netmiko.cisco.cisco_nxos_ssh import CiscoNxosSSH, CiscoNxosFileTransfer
 from netmiko.cisco.cisco_xr import CiscoXrSSH, CiscoXrTelnet, CiscoXrFileTransfer
 from netmiko.cisco.cisco_wlc_ssh import CiscoWlcSSH
+from netmiko.cisco.cisco_s200 import CiscoS200SSH
+from netmiko.cisco.cisco_s200 import CiscoS200Telnet
 from netmiko.cisco.cisco_s300 import CiscoS300SSH
+from netmiko.cisco.cisco_s300 import CiscoS300Telnet
 from netmiko.cisco.cisco_tp_tcce import CiscoTpTcCeSSH
+from netmiko.cisco.cisco_viptela import CiscoViptelaSSH
 
 __all__ = [
     "CiscoIosSSH",
     "CiscoIosTelnet",
     "CiscoAsaSSH",
+    "CiscoFtdSSH",
     "CiscoNxosSSH",
     "CiscoXrSSH",
     "CiscoXrTelnet",
     "CiscoWlcSSH",
+    "CiscoS200SSH",
+    "CiscoS200Telnet",
     "CiscoS300SSH",
+    "CiscoS300Telnet",
     "CiscoTpTcCeSSH",
+    "CiscoViptelaSSH",
     "CiscoIosBase",
     "CiscoIosFileTransfer",
     "InLineTransfer",
