@@ -1,30 +1,25 @@
 """Dell N2/3/4000 base driver- supports DNOS6."""
-from __future__ import unicode_literals
 from netmiko.dell.dell_powerconnect import DellPowerConnectBase
-import time
 
 
 class DellDNOS6Base(DellPowerConnectBase):
-    def session_preparation(self):
+    def session_preparation(self) -> None:
         """Prepare the session after the connection has been established."""
         self.ansi_escape_codes = True
-        self._test_channel_read()
+        self._test_channel_read(pattern=r"[>#]")
         self.set_base_prompt()
         self.enable()
-        self.disable_paging(command="terminal length 0")
         self.set_terminal_width()
-        # Clear the read buffer
-        time.sleep(0.3 * self.global_delay_factor)
-        self.clear_buffer()
+        self.disable_paging(command="terminal length 0")
 
     def save_config(
         self,
-        cmd="copy running-configuration startup-configuration",
-        confirm=False,
-        confirm_response="",
-    ):
+        cmd: str = "copy running-configuration startup-configuration",
+        confirm: bool = False,
+        confirm_response: str = "",
+    ) -> str:
         """Saves Config"""
-        return super(DellDNOS6Base, self).save_config(
+        return super().save_config(
             cmd=cmd, confirm=confirm, confirm_response=confirm_response
         )
 
