@@ -1,24 +1,23 @@
 """Extreme Virtual Services Platform Support."""
-from __future__ import print_function
-from __future__ import unicode_literals
-import time
 from netmiko.cisco_base_connection import CiscoSSHConnection
 
 
 class ExtremeVspSSH(CiscoSSHConnection):
     """Extreme Virtual Services Platform Support."""
 
-    def session_preparation(self):
+    def session_preparation(self) -> None:
         """Prepare the session after the connection has been established."""
-        self._test_channel_read()
+        self._test_channel_read(pattern=r"[>#]")
         self.set_base_prompt()
         self.disable_paging(command="terminal more disable")
-        # Clear the read buffer
-        time.sleep(0.3 * self.global_delay_factor)
-        self.clear_buffer()
 
-    def save_config(self, cmd="save config", confirm=False, confirm_response=""):
+    def save_config(
+        self,
+        cmd: str = "save config",
+        confirm: bool = False,
+        confirm_response: str = "",
+    ) -> str:
         """Save Config"""
-        return super(ExtremeVspSSH, self).save_config(
+        return super().save_config(
             cmd=cmd, confirm=confirm, confirm_response=confirm_response
         )
