@@ -20,9 +20,10 @@ SNMPDetect class defaults to SNMPv3
 Note, pysnmp is a required dependency for SNMPDetect and is intentionally not included in
 netmiko requirements. So installation of pysnmp might be required.
 """
-from __future__ import unicode_literals
-
+from typing import Optional, Dict, List
+from typing.re import Pattern
 import re
+import socket
 
 try:
     from pysnmp.entity.rfc3413.oneliner import cmdgen
@@ -30,7 +31,6 @@ except ImportError:
     raise ImportError("pysnmp not installed; please install it: 'pip install pysnmp'")
 
 from netmiko.ssh_dispatcher import CLASS_MAPPER
-from netmiko.py23_compat import text_type
 
 
 # Higher priority indicates a better match.
@@ -40,6 +40,11 @@ SNMP_MAPPER_BASE = {
         "expr": re.compile(r".*Arista Networks EOS.*", re.IGNORECASE),
         "priority": 99,
     },
+    "allied_telesis_awplus": {
+        "oid": ".1.3.6.1.2.1.1.1.0",
+        "expr": re.compile(r".*AlliedWare Plus.*", re.IGNORECASE),
+        "priority": 99,
+    },
     "paloalto_panos": {
         "oid": ".1.3.6.1.2.1.1.1.0",
         "expr": re.compile(r".*Palo Alto Networks.*", re.IGNORECASE),
@@ -47,7 +52,7 @@ SNMP_MAPPER_BASE = {
     },
     "hp_comware": {
         "oid": ".1.3.6.1.2.1.1.1.0",
-        "expr": re.compile(r".*HP Comware.*", re.IGNORECASE),
+        "expr": re.compile(r".*HP(E)? Comware.*", re.IGNORECASE),
         "priority": 99,
     },
     "hp_procurve": {
@@ -57,7 +62,7 @@ SNMP_MAPPER_BASE = {
     },
     "cisco_ios": {
         "oid": ".1.3.6.1.2.1.1.1.0",
-        "expr": re.compile(r".*Cisco IOS Software,.*", re.IGNORECASE),
+        "expr": re.compile(r".*Cisco IOS Software.*,.*", re.IGNORECASE),
         "priority": 60,
     },
     "cisco_xe": {
@@ -105,6 +110,21 @@ SNMP_MAPPER_BASE = {
         "expr": re.compile(r".*Juniper.*"),
         "priority": 99,
     },
+    "nokia_sros": {
+        "oid": ".1.3.6.1.2.1.1.1.0",
+        "expr": re.compile(r".*TiMOS.*"),
+        "priority": 99,
+    },
+    "dell_powerconnect": {
+        "oid": ".1.3.6.1.2.1.1.1.0",
+        "expr": re.compile(r"PowerConnect.*", re.IGNORECASE),
+        "priority": 50,
+    },
+    "mikrotik_routeros": {
+        "oid": ".1.3.6.1.2.1.1.1.0",
+        "expr": re.compile(r".*RouterOS.*", re.IGNORECASE),
+        "priority": 60,
+    },
 }
 
 # Ensure all SNMP device types are supported by Netmiko
@@ -115,6 +135,51 @@ for device_type in std_device_types:
         SNMP_MAPPER[device_type] = SNMP_MAPPER_BASE[device_type]
 
 
+def identify_address_type(entry: str) -> List[str]:
+    """
+    Return a list containing all ip types found. An empty list means no valid ip were found
+    Parameters
+    ----------
+    entry: str
+        Can be an ipv4, an ipv6 or an FQDN.
+
+    Returns
+    -------
+    list of string: list
+        A list of string 'IPv4' | 'IPv6' which indicates if entry is a valid ipv4 and/or ipv6.
+    """
+    try:
+        socket.inet_pton(socket.AF_INET, entry)
+        return ["IPv4"]
+    except socket.error:
+        pass
+
+    try:
+        socket.inet_pton(socket.AF_INET6, entry)
+        return ["IPv6"]
+    except socket.error:
+        pass
+
+    ip_types = []
+    try:
+        addrinfo = socket.getaddrinfo(entry, None)
+        for info in addrinfo:
+            ip = info[4][0]
+            try:
+                socket.inet_pton(socket.AF_INET, ip)
+                ip_types.append("IPv4")
+            except socket.error:
+                pass
+            try:
+                socket.inet_pton(socket.AF_INET6, ip)
+                ip_types.append("IPv6")
+            except socket.error:
+                pass
+    except socket.gaierror:
+        pass
+    return ip_types
+
+
 class SNMPDetect(object):
     """
     The SNMPDetect class tries to automatically determine the device type.
@@ -163,7 +228,6 @@ class SNMPDetect(object):
     encrypt_proto : str
         The SNMPv3 encryption protocol
 
-
     Methods
     -------
     autodetect()
@@ -173,17 +237,16 @@ class SNMPDetect(object):
 
     def __init__(
         self,
-        hostname,
-        snmp_version="v3",
-        snmp_port=161,
-        community=None,
-        user="",
-        auth_key="",
-        encrypt_key="",
-        auth_proto="sha",
-        encrypt_proto="aes128",
-    ):
-
+        hostname: str,
+        snmp_version: str = "v3",
+        snmp_port: int = 161,
+        community: Optional[str] = None,
+        user: str = "",
+        auth_key: str = "",
+        encrypt_key: str = "",
+        auth_proto: str = "sha",
+        encrypt_proto: str = "aes128",
+    ) -> None:
         # Check that the SNMP version is matching predefined type or raise ValueError
         if snmp_version == "v1" or snmp_version == "v2c":
             if not community:
@@ -228,9 +291,19 @@ class SNMPDetect(object):
         self.encrypt_key = encrypt_key
         self.auth_proto = self._snmp_v3_authentication[auth_proto]
         self.encryp_proto = self._snmp_v3_encryption[encrypt_proto]
-        self._response_cache = {}
+        self._response_cache: Dict[str, str] = {}
+        self.snmp_target = (self.hostname, self.snmp_port)
 
-    def _get_snmpv3(self, oid):
+        if "IPv6" in identify_address_type(self.hostname):
+            self.udp_transport_target = cmdgen.Udp6TransportTarget(
+                self.snmp_target, timeout=1.5, retries=2
+            )
+        else:
+            self.udp_transport_target = cmdgen.UdpTransportTarget(
+                self.snmp_target, timeout=1.5, retries=2
+            )
+
+    def _get_snmpv3(self, oid: str) -> str:
         """
         Try to send an SNMP GET operation using SNMPv3 for the specified OID.
 
@@ -244,7 +317,6 @@ class SNMPDetect(object):
         string : str
             The string as part of the value from the OID you are trying to retrieve.
         """
-        snmp_target = (self.hostname, self.snmp_port)
         cmd_gen = cmdgen.CommandGenerator()
 
         (error_detected, error_status, error_index, snmp_data) = cmd_gen.getCmd(
@@ -255,17 +327,17 @@ class SNMPDetect(object):
                 authProtocol=self.auth_proto,
                 privProtocol=self.encryp_proto,
             ),
-            cmdgen.UdpTransportTarget(snmp_target, timeout=1.5, retries=2),
+            self.udp_transport_target,
             oid,
             lookupNames=True,
             lookupValues=True,
         )
 
         if not error_detected and snmp_data[0][1]:
-            return text_type(snmp_data[0][1])
+            return str(snmp_data[0][1])
         return ""
 
-    def _get_snmpv2c(self, oid):
+    def _get_snmpv2c(self, oid: str) -> str:
         """
         Try to send an SNMP GET operation using SNMPv2 for the specified OID.
 
@@ -279,29 +351,28 @@ class SNMPDetect(object):
         string : str
             The string as part of the value from the OID you are trying to retrieve.
         """
-        snmp_target = (self.hostname, self.snmp_port)
         cmd_gen = cmdgen.CommandGenerator()
 
         (error_detected, error_status, error_index, snmp_data) = cmd_gen.getCmd(
             cmdgen.CommunityData(self.community),
-            cmdgen.UdpTransportTarget(snmp_target, timeout=1.5, retries=2),
+            self.udp_transport_target,
             oid,
             lookupNames=True,
             lookupValues=True,
         )
 
         if not error_detected and snmp_data[0][1]:
-            return text_type(snmp_data[0][1])
+            return str(snmp_data[0][1])
         return ""
 
-    def _get_snmp(self, oid):
+    def _get_snmp(self, oid: str) -> str:
         """Wrapper for generic SNMP call."""
         if self.snmp_version in ["v1", "v2c"]:
             return self._get_snmpv2c(oid)
         else:
             return self._get_snmpv3(oid)
 
-    def autodetect(self):
+    def autodetect(self) -> Optional[str]:
         """
         Try to guess the device_type using SNMP GET based on the SNMP_MAPPER dict. The type which
         is returned is directly matching the name in *netmiko.ssh_dispatcher.CLASS_MAPPER_BASE*
@@ -315,28 +386,30 @@ class SNMPDetect(object):
             The name of the device_type that must be running.
         """
         # Convert SNMP_MAPPER to a list and sort by priority
-        snmp_mapper_list = []
+        snmp_mapper_orig = []
         for k, v in SNMP_MAPPER.items():
-            snmp_mapper_list.append({k: v})
+            snmp_mapper_orig.append({k: v})
         snmp_mapper_list = sorted(
-            snmp_mapper_list, key=lambda x: list(x.values())[0]["priority"]
+            snmp_mapper_orig, key=lambda x: list(x.values())[0]["priority"]  # type: ignore
         )
         snmp_mapper_list.reverse()
 
         for entry in snmp_mapper_list:
             for device_type, v in entry.items():
-                oid = v["oid"]
-                regex = v["expr"]
+                oid: str = v["oid"]  # type: ignore
+                regex: Pattern = v["expr"]
 
-            # Used cache data if we already queryied this OID
-            if self._response_cache.get(oid):
-                snmp_response = self._response_cache.get(oid)
-            else:
-                snmp_response = self._get_snmp(oid)
-                self._response_cache[oid] = snmp_response
+                # Used cache data if we already queryied this OID
+                if self._response_cache.get(oid):
+                    snmp_response = self._response_cache.get(oid)
+                else:
+                    snmp_response = self._get_snmp(oid)
+                    self._response_cache[oid] = snmp_response
 
-            # See if we had a match
-            if re.search(regex, snmp_response):
-                return device_type
+                # See if we had a match
+                assert isinstance(snmp_response, str)
+                if re.search(regex, snmp_response):
+                    assert isinstance(device_type, str)
+                    return device_type
 
         return None
