@@ -10,7 +10,7 @@ The **SSHDetect** class is instantiated using the same parameters than a standar
 connection (see the *netmiko.ssh_dispatacher.ConnectHandler* function). The only acceptable value
 for the 'device_type' argument is 'autodetect'.
 
-The auto-detection is solely based on the *SSH_MAPPER_BASE* dictionary. The keys are the name of
+The auto-detection is solely based on *SSH_MAPPER_BASE*. The keys are the name of
 the 'device_type' supported for auto-detection and the value is another dictionary describing how
 to handle the auto-detection.
 
@@ -38,10 +38,12 @@ Examples
 >>> remote_device['device_type'] = best_match
 >>> connection = ConnectHandler(**remote_device)
 """
-from __future__ import unicode_literals
-
+from typing import Any, List, Optional, Union, Dict
 import re
 import time
+
+import paramiko
+
 from netmiko.ssh_dispatcher import ConnectHandler
 from netmiko.base_connection import BaseConnection
 
@@ -49,7 +51,7 @@ from netmiko.base_connection import BaseConnection
 # 'dispatch' key is the SSHDetect method to call. dispatch key will be popped off dictionary
 # remaining keys indicate kwargs that will be passed to dispatch method.
 # Note, the 'cmd' needs to avoid output paging.
-SSH_MAPPER_BASE = {
+SSH_MAPPER_DICT = {
     "alcatel_aos": {
         "cmd": "show system",
         "search_patterns": [r"Alcatel-Lucent"],
@@ -62,6 +64,12 @@ SSH_MAPPER_BASE = {
         "priority": 99,
         "dispatch": "_autodetect_std",
     },
+    "allied_telesis_awplus": {
+        "cmd": "show version",
+        "search_patterns": ["AlliedWare Plus"],
+        "priority": 99,
+        "dispatch": "_autodetect_std",
+    },
     "apresia_aeos": {
         "cmd": "show system",
         "search_patterns": ["Apresia"],
@@ -70,7 +78,13 @@ SSH_MAPPER_BASE = {
     },
     "arista_eos": {
         "cmd": "show version",
-        "search_patterns": [r"Arista"],
+        "search_patterns": [r"Arista", r"vEOS"],
+        "priority": 99,
+        "dispatch": "_autodetect_std",
+    },
+    "ciena_saos": {
+        "cmd": "software show",
+        "search_patterns": [r"saos"],
         "priority": 99,
         "dispatch": "_autodetect_std",
     },
@@ -89,6 +103,12 @@ SSH_MAPPER_BASE = {
         "priority": 99,
         "dispatch": "_autodetect_std",
     },
+    "cisco_xe": {
+        "cmd": "show version",
+        "search_patterns": [r"Cisco IOS XE Software"],
+        "priority": 99,
+        "dispatch": "_autodetect_std",
+    },
     "cisco_nxos": {
         "cmd": "show version",
         "search_patterns": [r"Cisco Nexus Operating System", r"NX-OS"],
@@ -101,15 +121,36 @@ SSH_MAPPER_BASE = {
         "priority": 99,
         "dispatch": "_autodetect_std",
     },
+    "cisco_xr_2": {
+        "cmd": "show version brief",
+        "search_patterns": [r"Cisco IOS XR"],
+        "priority": 99,
+        "dispatch": "_autodetect_std",
+    },
     "dell_force10": {
         "cmd": "show version",
-        "search_patterns": [r"S4048-ON"],
+        "search_patterns": [r"Real Time Operating System Software"],
+        "priority": 99,
+        "dispatch": "_autodetect_std",
+    },
+    "dell_os9": {
+        "cmd": "show system",
+        "search_patterns": [
+            r"Dell Application Software Version:  9",
+            r"Dell Networking OS Version : 9",
+        ],
         "priority": 99,
         "dispatch": "_autodetect_std",
     },
     "dell_os10": {
         "cmd": "show version",
-        "search_patterns": [r"Dell EMC Networking OS10-Enterprise"],
+        "search_patterns": [r"Dell EMC Networking OS10.Enterprise"],
+        "priority": 99,
+        "dispatch": "_autodetect_std",
+    },
+    "dell_powerconnect": {
+        "cmd": "show system",
+        "search_patterns": [r"PowerConnect"],
         "priority": 99,
         "dispatch": "_autodetect_std",
     },
@@ -125,6 +166,18 @@ SSH_MAPPER_BASE = {
         "priority": 99,
         "dispatch": "_autodetect_std",
     },
+    "hp_comware": {
+        "cmd": "display version",
+        "search_patterns": ["HPE Comware", "HP Comware"],
+        "priority": 99,
+        "dispatch": "_autodetect_std",
+    },
+    "hp_procurve": {
+        "cmd": "show version",
+        "search_patterns": [r"Image stamp.*/code/build"],
+        "priority": 99,
+        "dispatch": "_autodetect_std",
+    },
     "huawei": {
         "cmd": "display version",
         "search_patterns": [
@@ -151,15 +204,35 @@ SSH_MAPPER_BASE = {
         "priority": 99,
         "dispatch": "_autodetect_std",
     },
-    "brocade_netiron": {
+    "ericsson_ipos": {
         "cmd": "show version",
-        "search_patterns": [r"NetIron"],
+        "search_patterns": [r"Ericsson IPOS Version"],
+        "priority": 99,
+        "dispatch": "_autodetect_std",
+    },
+    "extreme_exos": {
+        "cmd": "show version",
+        "search_patterns": [r"ExtremeXOS"],
+        "priority": 99,
+        "dispatch": "_autodetect_std",
+    },
+    "extreme_netiron": {
+        "cmd": "show version",
+        "search_patterns": [r"(NetIron|MLX)"],
         "priority": 99,
         "dispatch": "_autodetect_std",
     },
     "extreme_slx": {
         "cmd": "show version",
-        "search_patterns": [r"SLX-OS Operating System Software"],
+        "search_patterns": [
+            r"SLX-OS Operating System",
+        ],
+        "priority": 99,
+        "dispatch": "_autodetect_std",
+    },
+    "extreme_tierra": {
+        "cmd": "show version",
+        "search_patterns": [r"TierraOS Software"],
         "priority": 99,
         "dispatch": "_autodetect_std",
     },
@@ -169,8 +242,83 @@ SSH_MAPPER_BASE = {
         "priority": 99,
         "dispatch": "_autodetect_std",
     },
+    "cisco_wlc": {
+        "cmd": "",
+        "dispatch": "_autodetect_remote_version",
+        "search_patterns": [r"CISCO_WLC"],
+        "priority": 99,
+    },
+    "cisco_wlc_85": {
+        "cmd": "show inventory",
+        "dispatch": "_autodetect_std",
+        "search_patterns": [r"Cisco.*Wireless.*Controller"],
+        "priority": 99,
+    },
+    "mellanox_mlnxos": {
+        "cmd": "show version",
+        "search_patterns": [r"Onyx", r"SX_PPC_M460EX"],
+        "priority": 99,
+        "dispatch": "_autodetect_std",
+    },
+    "yamaha": {
+        "cmd": "show copyright",
+        "search_patterns": [r"Yamaha Corporation"],
+        "priority": 99,
+        "dispatch": "_autodetect_std",
+    },
+    "fortinet": {
+        "cmd": "get system status",
+        "search_patterns": [r"FortiOS", r"FortiGate"],
+        "priority": 99,
+        "dispatch": "_autodetect_std",
+    },
+    "paloalto_panos": {
+        "cmd": "show system info",
+        "search_patterns": [r"model:\s+PA"],
+        "priority": 99,
+        "dispatch": "_autodetect_std",
+    },
+    "supermicro_smis": {
+        "cmd": "show system info",
+        "search_patterns": [r"Super Micro Computer"],
+        "priority": 99,
+        "dispatch": "_autodetect_std",
+    },
+    "flexvnf": {
+        "cmd": "show system package-info",
+        "search_patterns": [r"Versa FlexVNF"],
+        "priority": 99,
+        "dispatch": "_autodetect_std",
+    },
+    "cisco_viptela": {
+        "cmd": "show system status",
+        "search_patterns": [r"Viptela, Inc"],
+        "priority": 99,
+        "dispatch": "_autodetect_std",
+    },
+    "oneaccess_oneos": {
+        "cmd": "show version",
+        "search_patterns": [r"OneOS"],
+        "priority": 99,
+        "dispatch": "_autodetect_std",
+    },
 }
 
+# Sort SSH_MAPPER_DICT such that the most common commands are first
+cmd_count: Dict[str, int] = {}
+for k, v in SSH_MAPPER_DICT.items():
+    my_cmd = v["cmd"]
+    assert isinstance(my_cmd, str)
+    count = cmd_count.setdefault(my_cmd, 0)
+    cmd_count[my_cmd] = count + 1
+cmd_count = {k: v for k, v in sorted(cmd_count.items(), key=lambda item: item[1])}
+
+# SSH_MAPPER_BASE is a list
+SSH_MAPPER_BASE = sorted(
+    SSH_MAPPER_DICT.items(), key=lambda item: int(cmd_count[str(item[1]["cmd"])])
+)
+SSH_MAPPER_BASE.reverse()
+
 
 class SSHDetect(object):
     """
@@ -199,20 +347,26 @@ class SSHDetect(object):
         Try to determine the device type.
     """
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         """
         Constructor of the SSHDetect class
         """
         if kwargs["device_type"] != "autodetect":
             raise ValueError("The connection device_type must be 'autodetect'")
+        # Always set cmd_verify to False for autodetect
+        kwargs["global_cmd_verify"] = False
         self.connection = ConnectHandler(*args, **kwargs)
+
+        # Add additional sleep to let the login complete.
+        time.sleep(3)
+
         # Call the _test_channel_read() in base to clear initial data
         output = BaseConnection._test_channel_read(self.connection)
         self.initial_buffer = output
-        self.potential_matches = {}
-        self._results_cache = {}
+        self.potential_matches: Dict[str, int] = {}
+        self._results_cache: Dict[str, str] = {}
 
-    def autodetect(self):
+    def autodetect(self) -> Union[str, None]:
         """
         Try to guess the best 'device_type' based on patterns defined in SSH_MAPPER_BASE
 
@@ -221,9 +375,10 @@ class SSHDetect(object):
         best_match : str or None
             The device type that is currently the best to use to interact with the device
         """
-        for device_type, autodetect_dict in SSH_MAPPER_BASE.items():
+        for device_type, autodetect_dict in SSH_MAPPER_BASE:
             tmp_dict = autodetect_dict.copy()
             call_method = tmp_dict.pop("dispatch")
+            assert isinstance(call_method, str)
             autodetect_method = getattr(self, call_method)
             accuracy = autodetect_method(**tmp_dict)
             if accuracy:
@@ -232,6 +387,12 @@ class SSHDetect(object):
                     best_match = sorted(
                         self.potential_matches.items(), key=lambda t: t[1], reverse=True
                     )
+                    # WLC needs two different auto-dectect solutions
+                    if "cisco_wlc_85" in best_match[0]:
+                        best_match[0] = ("cisco_wlc", 99)
+                    # IOS XR needs two different auto-dectect solutions
+                    if "cisco_xr_2" in best_match[0]:
+                        best_match[0] = ("cisco_xr", 99)
                     self.connection.disconnect()
                     return best_match[0][0]
 
@@ -245,7 +406,7 @@ class SSHDetect(object):
         self.connection.disconnect()
         return best_match[0][0]
 
-    def _send_command(self, cmd=""):
+    def _send_command(self, cmd: str = "") -> str:
         """
         Handle reading/writing channel directly. It is also sanitizing the output received.
 
@@ -261,12 +422,11 @@ class SSHDetect(object):
         """
         self.connection.write_channel(cmd + "\n")
         time.sleep(1)
-        output = self.connection._read_channel_timing()
-        output = self.connection.strip_ansi_escape_codes(output)
+        output = self.connection.read_channel_timing(last_read=6.0)
         output = self.connection.strip_backspaces(output)
         return output
 
-    def _send_command_wrapper(self, cmd):
+    def _send_command_wrapper(self, cmd: str) -> str:
         """
         Send command to the remote device with a caching feature to avoid sending the same command
         twice based on the SSH_MAPPER_BASE dict cmd key.
@@ -289,7 +449,56 @@ class SSHDetect(object):
         else:
             return cached_results
 
-    def _autodetect_std(self, cmd="", search_patterns=None, re_flags=re.I, priority=99):
+    def _autodetect_remote_version(
+        self,
+        search_patterns: Optional[List[str]] = None,
+        re_flags: int = re.IGNORECASE,
+        priority: int = 99,
+        **kwargs: Any
+    ) -> int:
+        """
+        Method to try auto-detect the device type, by matching a regular expression on the reported
+        remote version of the SSH server.
+
+        Parameters
+        ----------
+        search_patterns : list
+            A list of regular expression to look for in the reported remote SSH version
+            (default: None).
+        re_flags: re.flags, optional
+            Any flags from the python re module to modify the regular expression (default: re.I).
+        priority: int, optional
+            The confidence the match is right between 0 and 99 (default: 99).
+        """
+        invalid_responses = [r"^$"]
+
+        if not search_patterns:
+            return 0
+
+        try:
+            remote_conn = self.connection.remote_conn
+            assert isinstance(remote_conn, paramiko.Channel)
+            assert remote_conn.transport is not None
+            remote_version = remote_conn.transport.remote_version
+            for pattern in invalid_responses:
+                match = re.search(pattern, remote_version, flags=re.I)
+                if match:
+                    return 0
+            for pattern in search_patterns:
+                match = re.search(pattern, remote_version, flags=re_flags)
+                if match:
+                    return priority
+        except Exception:
+            return 0
+        return 0
+
+    def _autodetect_std(
+        self,
+        cmd: str = "",
+        search_patterns: Optional[List[str]] = None,
+        re_flags: int = re.IGNORECASE,
+        priority: int = 99,
+    ) -> int:
         """
         Standard method to try to auto-detect the device type. This method will be called for each
         device_type present in SSH_MAPPER_BASE dict ('dispatch' key). It will attempt to send a
@@ -314,6 +523,7 @@ class SSHDetect(object):
             r"%Error",
             r"command not found",
             r"Syntax Error: unexpected argument",
+            r"% Unrecognized command found at",
         ]
         if not cmd or not search_patterns:
             return 0
