@@ -1,5 +1,6 @@
 """Generic Terminal Server driver."""
-from __future__ import unicode_literals
+from typing import Any
+
 from netmiko.base_connection import BaseConnection
 
 
@@ -9,16 +10,24 @@ class TerminalServer(BaseConnection):
     Allow direct write_channel / read_channel operations without session_preparation causing
     an exception.
     """
-    def session_preparation(self):
+
+    def session_preparation(self) -> None:
         """Do nothing here; base_prompt is not set; paging is not disabled."""
         pass
 
 
 class TerminalServerSSH(TerminalServer):
     """Generic Terminal Server driver SSH."""
+
     pass
 
 
 class TerminalServerTelnet(TerminalServer):
     """Generic Terminal Server driver telnet."""
-    pass
+
+    def telnet_login(self, *args: Any, **kwargs: Any) -> str:
+        # Disable automatic handling of username and password when using terminal server driver
+        return ""
+
+    def std_login(self, *args: Any, **kwargs: Any) -> str:
+        return super().telnet_login(*args, **kwargs)
